@@ -83,6 +83,19 @@ class VerticalSwing(IntEnum):
     SwingLower = 11
 
 
+def generate_temperature_record(temp_f):
+    temSet = round((temp_f - 32.0) * 5.0 / 9.0)
+    temRec = (int)((((temp_f - 32.0) * 5.0 / 9.0) - temSet) > 0)
+    return {"f": temp_f, "temSet": temSet, "temRec": temRec}
+
+
+TEMP_MIN = 16
+TEMP_MAX = 30
+TEMP_MIN_F = 60
+TEMP_MAX_F = 86
+TEMP_TABLE = [generate_temperature_record(x) for x in range(TEMP_MIN_F, TEMP_MAX_F + 1)]
+
+
 class DeviceInfo:
     """Device information class, used to identify and connect
 
@@ -159,8 +172,8 @@ class Device:
         self.device_key = None
 
         """ Device properties """
-        self._hid = None
-        self._version = None
+        self.hid = None
+        self.version = None
         self._properties = None
         self._dirty = []
 
@@ -206,15 +219,23 @@ class Device:
     async def request_version(self) -> None:
         """Request the firmware version from the device."""
         ret = await network.request_state(["hid"], self.device_info, self.device_key)
-        self._hid = ret.get("hid")
+        self.hid = ret.get("hid")
 
         # Ex: hid = 362001000762+U-CS532AE(LT)V3.31.bin
-        if self._hid:
-            match = re.search(r"(?<=V)([\d.]+)\.bin$", self._hid)
-            self._version = match and match.group(1)
+        if self.hid:
+            match = re.search(r"(?<=V)([\d.]+)\.bin$", self.hid)
+            self.version = match and match.group(1)
+
+            # Special case firmwares ...
+            if (
+                self.hid.endswith("_JDV1.bin")
+                or self.hid.endswith("(MTK)V1.bin")
+                or self.hid.endswith("362001000967V2.bin")
+            ):
+                self.version = "4.0"
 
     async def update_state(self):
-        """ Update the internal state of the device structure of the physical device """
+        """Update the internal state of the device structure of the physical device"""
         if not self.device_key:
             await self.bind()
 
@@ -223,7 +244,7 @@ class Device:
         props = [x.value for x in Props]
 
         try:
-            if not self._hid:
+            if not self.hid:
                 await self.request_version()
 
             self._properties = await network.request_state(
@@ -233,7 +254,7 @@ class Device:
             raise DeviceTimeoutError
 
     async def push_state_update(self):
-        """ Push any pending state updates to the unit """
+        """Push any pending state updates to the unit"""
         if not self._dirty:
             return
 
@@ -247,6 +268,11 @@ class Device:
             value = self._properties.get(name)
             self._logger.debug("Sending remote state update %s -> %s", name, value)
             props[name] = value
+            if name == Props.TEMP_SET.value:
+                props[Props.TEMP_BIT.value] = self._properties.get(Props.TEMP_BIT.value)
+                props[Props.TEMP_UNIT.value] = self._properties.get(
+                    Props.TEMP_UNIT.value
+                )
 
         self._dirty.clear()
 
@@ -256,13 +282,13 @@ class Device:
             raise DeviceTimeoutError
 
     def get_property(self, name):
-        """ Generic lookup of properties tracked from the physical device """
+        """Generic lookup of properties tracked from the physical device"""
         if self._properties:
             return self._properties.get(name.value)
         return None
 
     def set_property(self, name, value):
-        """ Generic setting of properties for the physical device """
+        """Generic setting of properties for the physical device"""
         if not self._properties:
             self._properties = {}
 
@@ -289,13 +315,33 @@ class Device:
     def mode(self, value: int):
         self.set_property(Props.MODE, int(value))
 
+    def _convert_to_units(self, value, bit):
+        if self.temperature_units != TemperatureUnits.F.value:
+            return value
+
+        f = next(t for t in TEMP_TABLE if t["temSet"] == value and t["temRec"] == bit)
+        return f["f"]
+
     @property
     def target_temperature(self) -> int:
-        return self.get_property(Props.TEMP_SET)
+        temSet = self.get_property(Props.TEMP_SET)
+        temRec = self.get_property(Props.TEMP_BIT)
+        return self._convert_to_units(temSet, temRec)
 
     @target_temperature.setter
     def target_temperature(self, value: int):
-        self.set_property(Props.TEMP_SET, int(value))
+        def validate(val):
+            if val > TEMP_MAX or val < TEMP_MIN:
+                raise ValueError(f"Specified temperature {val} is out of range.")
+
+        if self.temperature_units == 1:
+            rec = generate_temperature_record(value)
+            validate(rec["temSet"])
+            self.set_property(Props.TEMP_SET, rec["temSet"])
+            self.set_property(Props.TEMP_BIT, rec["temRec"])
+        else:
+            validate(value)
+            self.set_property(Props.TEMP_SET, int(value))
 
     @property
     def temperature_units(self) -> int:
@@ -308,12 +354,13 @@ class Device:
     @property
     def current_temperature(self) -> int:
         prop = self.get_property(Props.TEMP_SENSOR)
+        bit = self.get_property(Props.TEMP_BIT)
         if prop is not None:
-            v = self._version and int(self._version.split(".")[0])
+            v = self.version and int(self.version.split(".")[0])
             if v == 4:
-                return prop
-
-            return prop - 40
+                return self._convert_to_units(prop, bit)
+            elif prop != 0:
+                return self._convert_to_units(prop - 40, bit)
 
         return self.target_temperature
 
